package sun.security.jca;

import java.util.NoSuchElementException;
import java.util.Iterator;
import java.security.Security;
import java.util.AbstractList;
import java.util.ArrayList;
import java.util.Arrays;
import java.security.AccessController;
import java.security.PrivilegedAction;
import java.util.List;
import java.security.Provider;
import sun.security.util.Debug;

public final class ProviderList
{
    static final Debug debug;
    private static final ProviderConfig[] PC0;
    private static final Provider[] P0;
    static final ProviderList EMPTY;
    private static final Provider EMPTY_PROVIDER;
    private final ProviderConfig[] configs;
    private volatile boolean allLoaded;
    private final List<Provider> userList;
    
    static ProviderList fromSecurityProperties() {
        return (ProviderList)AccessController.doPrivileged(new PrivilegedAction<ProviderList>() {
            @Override
            public ProviderList run() {
                return new ProviderList();
            }
        });
    }
    
    public static ProviderList add(ProviderList list, Provider provider) {
        return insertAt(list, provider, -1);
    }
    
    public static ProviderList insertAt(ProviderList list, Provider provider, int n) {
        if (list.getProvider(provider.getName()) != null) {
            return list;
        }
        ArrayList list2 = new ArrayList(Arrays.asList(list.configs));
        int size = list2.size();
        if (n < 0 || n > size) {
            n = size;
        }
        list2.add(n, new ProviderConfig(provider));
        return new ProviderList((ProviderConfig[])list2.toArray(ProviderList.PC0), true);
    }
    
    public static ProviderList remove(ProviderList list, String anObject) {
        if (list.getProvider(anObject) == null) {
            return list;
        }
        ProviderConfig[] array = new ProviderConfig[list.size() - 1];
        int n = 0;
        for (ProviderConfig providerConfig : list.configs) {
            if (!providerConfig.getProvider().getName().equals(anObject)) {
                array[n++] = providerConfig;
            }
        }
        return new ProviderList(array, true);
    }
    
    public static ProviderList newList(Provider... array) {
        ProviderConfig[] array2 = new ProviderConfig[array.length];
        for (int i = 0; i < array.length; ++i) {
            array2[i] = new ProviderConfig(array[i]);
        }
        return new ProviderList(array2, true);
    }
    
    private ProviderList(ProviderConfig[] configs, boolean allLoaded) {
        this.userList = new AbstractList<Provider>() {
            @Override
            public int size() {
                return ProviderList.this.configs.length;
            }
            
            @Override
            public Provider get(int n) {
                return ProviderList.this.getProvider(n);
            }
        };
        this.configs = configs;
        this.allLoaded = allLoaded;
    }
    
    private ProviderList() {
        this.userList = new AbstractList<Provider>() {
            @Override
            public int size() {
                return ProviderList.this.configs.length;
            }
            
            @Override
            public Provider get(int n) {
                return ProviderList.this.getProvider(n);
            }
        };
        ArrayList<Object> obj = new ArrayList();
        int n = 1;
        while (true) {
            String property = Security.getProperty("security.provider." + n);
            if (property == null) {
                break;
            }
            String trim = property.trim();
            if (trim.length() == 0) {
                System.err.println("invalid entry for security.provider." + n);
                break;
            }
            int index = trim.indexOf(32);
            Object o;
            if (index == -1) {
                o = new ProviderConfig(trim);
            }
            else {
                o = new ProviderConfig(trim.substring(0, index), trim.substring(index + 1).trim());
            }
            if (!obj.contains(o)) {
                obj.add(o);
            }
            ++n;
        }
        this.configs = (ProviderConfig[])obj.toArray(ProviderList.PC0);
        if (ProviderList.debug != null) {
            ProviderList.debug.println("provider configuration: " + obj);
        }
    }
    
    ProviderList getJarList(String[] array) {
        ArrayList<Object> list = new ArrayList();
        for (int length = array.length, i = 0; i < length; ++i) {
            ProviderConfig o = new ProviderConfig(array[i]);
            ProviderConfig[] configs = this.configs;
            int length2 = configs.length;
            int j = 0;
            while (j < length2) {
                ProviderConfig providerConfig = configs[j];
                if (providerConfig.equals(o)) {
                    o = providerConfig;
                    break;
                }
                ++j;
            }
            list.add(o);
        }
        return new ProviderList((ProviderConfig[])list.toArray(ProviderList.PC0), false);
    }
    
    public int size() {
        return this.configs.length;
    }
    
    Provider getProvider(int n) {
        Provider provider = this.configs[n].getProvider();
        return (provider != null) ? provider : ProviderList.EMPTY_PROVIDER;
    }
    
    public List<Provider> providers() {
        return this.userList;
    }
    
    private ProviderConfig getProviderConfig(String s) {
        int index = this.getIndex(s);
        return (index != -1) ? this.configs[index] : null;
    }
    
    public Provider getProvider(String s) {
        ProviderConfig providerConfig = this.getProviderConfig(s);
        return (providerConfig == null) ? null : providerConfig.getProvider();
    }
    
    public int getIndex(String anObject) {
        int i = 0;
        while (i < this.configs.length) {
            if (this.getProvider(i).getName().equals(anObject)) {
                return i;
            }
            ++i;
        }
        return -1;
    }
    
    private int loadAll() {
        if (this.allLoaded) {
            return this.configs.length;
        }
        if (ProviderList.debug != null) {
            ProviderList.debug.println("Loading all providers");
            new Exception("Debug Info. Call trace:").printStackTrace();
        }
        int n = 0;
        for (int i = 0; i < this.configs.length; ++i) {
            if (this.configs[i].getProvider() != null) {
                ++n;
            }
        }
        if (n == this.configs.length) {
            this.allLoaded = true;
        }
        return n;
    }
    
    ProviderList removeInvalid() {
        int loadAll = this.loadAll();
        if (loadAll == this.configs.length) {
            return this;
        }
        ProviderConfig[] array = new ProviderConfig[loadAll];
        int i = 0;
        int n = 0;
        while (i < this.configs.length) {
            ProviderConfig providerConfig = this.configs[i];
            if (providerConfig.isLoaded()) {
                array[n++] = providerConfig;
            }
            ++i;
        }
        return new ProviderList(array, true);
    }
    
    public Provider[] toArray() {
        return (Provider[])this.providers().toArray(ProviderList.P0);
    }
    
    @Override
    public String toString() {
        return Arrays.asList(this.configs).toString();
    }
    
    public Provider.Service getService(String type, String algorithm) {
        int i = 0;
        while (i < this.configs.length) {
            Provider.Service service = this.getProvider(i).getService(type, algorithm);
            if (service != null) {
                return service;
            }
            ++i;
        }
        return null;
    }
    
    public List<Provider.Service> getServices(String s, String s2) {
        return new ProviderList.ServiceList(s, s2);
    }
    
    @Deprecated
    public List<Provider.Service> getServices(String s, List<String> list) {
        ArrayList<ServiceId> list2 = new ArrayList();
        Iterator<Object> iterator = list.iterator();
        while (iterator.hasNext()) {
            list2.add(new ServiceId(s, (String)iterator.next()));
        }
        return this.getServices(list2);
    }
    
    public List<Provider.Service> getServices(List<ServiceId> list) {
        return new ProviderList.ServiceList(list);
    }
    
    ProviderList(ProviderList.1 provider) {
        this();
    }
    
    static {
        debug = Debug.getInstance("jca", "ProviderList");
        PC0 = new ProviderConfig[0];
        P0 = new Provider[0];
        EMPTY = new ProviderList(ProviderList.PC0, true);
        EMPTY_PROVIDER = new Provider("##Empty##", 1.0, "initialization in progress") {
            private static final long serialVersionUID = 1151354171352296389L;
            
            @Override
            public Provider.Service getService(String s, String s2) {
                return null;
            }
        };
    }
    
    private final class ServiceList extends AbstractList<Provider.Service>
    {
        private final String type;
        private final String algorithm;
        private final List<ServiceId> ids;
        private Provider.Service firstService;
        private List<Provider.Service> services;
        private int providerIndex;
        
        ServiceList(String type, String algorithm) {
            this.type = type;
            this.algorithm = algorithm;
            this.ids = null;
        }
        
        ServiceList(List<ServiceId> ids) {
            this.type = null;
            this.algorithm = null;
            this.ids = ids;
        }
        
        private void addService(Provider.Service firstService) {
            if (this.firstService == null) {
                this.firstService = firstService;
            }
            else {
                if (this.services == null) {
                    (this.services = new ArrayList(4)).add(this.firstService);
                }
                this.services.add(firstService);
            }
        }
        
        private Provider.Service tryGet(int n) {
            while (n != 0 || this.firstService == null) {
                if (this.services != null && this.services.size() > n) {
                    return (Provider.Service)this.services.get(n);
                }
                if (this.providerIndex >= ProviderList.this.configs.length) {
                    return null;
                }
                Provider provider = ProviderList.this.getProvider(this.providerIndex++);
                if (this.type != null) {
                    Provider.Service service2 = provider.getService(this.type, this.algorithm);
                    if (service2 != null) {
                        this.addService(service2);
                    }
                    else {
                        continue;
                    }
                }
                else {
                    for (ServiceId serviceId : this.ids) {
                        Provider.Service service = provider.getService(serviceId.type, serviceId.algorithm);
                        if (service != null) {
                            this.addService(service);
                        }
                        else {
                            continue;
                        }
                    }
                }
            }
            return this.firstService;
        }
        
        @Override
        public Provider.Service get(int n) {
            Provider.Service tryGet = this.tryGet(n);
            if (tryGet == null) {
                throw new IndexOutOfBoundsException();
            }
            return tryGet;
        }
        
        @Override
        public int size() {
            int size;
            if (this.services != null) {
                size = this.services.size();
            }
            else {
                size = ((this.firstService != null) ? 1 : 0);
            }
            while (this.tryGet(size) != null) {
                ++size;
            }
            return size;
        }
        
        @Override
        public boolean isEmpty() {
            return this.tryGet(0) == null;
        }
        
        @Override
        public Iterator<Provider.Service> iterator() {
            return new Iterator<Provider.Service>() {
                int index;
                
                @Override
                public boolean hasNext() {
                    return ProviderList.ServiceList.access$200(ProviderList.ServiceList.this, this.index) != null;
                }
                
                @Override
                public Provider.Service next() {
                    Provider.Service access$200 = ProviderList.ServiceList.access$200(ProviderList.ServiceList.this, this.index);
                    if (access$200 == null) {
                        throw new NoSuchElementException();
                    }
                    ++this.index;
                    return access$200;
                }
                
                @Override
                public void remove() {
                    throw new UnsupportedOperationException();
                }
            };
        }
        
        static Provider.Service access$200(ProviderList.ServiceList list, int n) {
            return list.tryGet(n);
        }
    }
}
